# Load libraries and user defined functions

# Use:
# install.packages("name")
# To install "name" package

library(sjmisc) # for string manipulation
library(stringi) # for string manipulation
library(DescTools) # for the CCC
library(plyr) # for the progress bar
library(randomForest) # for Random Forests
library(pwr) # for power calculations
library(MESS) # for power calculations
library(extrafont) # to load fonts
#font_import() # run only the first time
loadfonts(device="win") 

# Function for logit normalization
my.logit<-function(x){
  x<-ifelse(x<10^-16,10^-16,x)
  x<-ifelse(x>1-10^-16,1-10^-16,x)
  log(x/(1-x))  
}

# Function for Bonferroni adjustment of a p-value vector
bonferroni<-function(x){p.adjust(x,"bonferroni")} 

# Load the data (already matched by column)
load("alteration.RData")
load("response.RData")
dim(alteration) # 48270 alterations, 648 cell lines
dim(response) # 265 compounds, 648 cell lines

# Verify vertical matching
all(colnames(alteration)==colnames(response))

# Frequency and proportion of each alteration
freq.alterations<-apply(alteration,1,sum)
prop.alterations<-apply(alteration,1,mean)
# Reorder alterations by their frequency
alteration<-alteration[order(prop.alterations),]
freq.alterations<-sort(freq.alterations)
prop.alterations<-sort(prop.alterations)

# Proportion of alterations present in each cell line
prop.lines<-apply(alteration,2,mean)
# Reorder cell lines by number of present alterations
alteration<-alteration[,order(prop.lines)]
response<-response[,order(prop.lines)]
prop.lines<-sort(prop.lines)
# Verify vertical matching
all(colnames(alteration)==colnames(response))

# Non-missing cell line's AUC (sample sizes)
sample.sizes<-apply(response,1,function(x){sum(!is.na(x))})
trunc(mean(sample.sizes)) # mean sample size

# Reorder compounds by sample size
response<-response[order(sample.sizes),]
sample.sizes<-sort(sample.sizes)

# Figure 1
to.find<-c(0,1,5,95)
positions<-match(to.find,trunc(100*prop.alterations))
prop.alterations[positions]
no.alterations<-sum(prop.alterations==0)

{
tiff(width=0.9*8,height=5*2,"Figure 1.tiff", units = "in", res = 800, compression ="lzw")
par(family="Palatino Linotype")
par(mfrow=c(2,1))
par(mar=c(5,5,2,2))
cex<-0.1
plot(cbind(no.alterations+1,
           which(alteration[no.alterations+1,]==1)),
     pch=16,cex.lab=1.6,col=gray(0),
     xlim=c(1,nrow(alteration)),ylim=c(1,ncol(alteration)),cex=cex,
     xlab="Alteration ID",
     ylab="Cellular line",axes=FALSE)
box()
axis(side=1,at=c(0,seq(10000,50000,by=10000)),
     labels=c(0,"10,000","20,000","30,000","40,000","50,000"),
     cex.axis=1.25)
axis(side=2,cex.axis=1.25)
par(xpd=TRUE)
text(positions,par()$usr[4]+35,
     labels=paste0(freq.alterations[positions],
                   " (",to.find,"%)"),cex=1.3)
for(i in (no.alterations+2):nrow(alteration))
  points(cbind(i,which(alteration[i,]==1)),pch=16,cex=cex,col=gray(0))
points(cbind(positions,par()$usr[4]),pch="|")
text(-8000,700,labels="(A)",font=2,cex=1.6)

dim(response)
missings<-is.na(response)
cex<-0.3
plot(cbind(1,which(missings[1,]==1)),
     pch=16,col=gray(0.6),
     xlim=c(1,nrow(missings)),ylim=c(1,ncol(missings)),cex=cex,
     xlab="Compound ID",
     ylab="Cellular line",cex.axis=1.25,cex.lab=1.6)
par(xpd=TRUE)
for(i in 2:nrow(missings))
  points(cbind(i,which(missings[i,]==1)),pch=16,cex=cex,col=gray(0.6))
text(c(1,nrow(missings)),par()$usr[4]+35,
     labels=paste0(apply(missings,1,sum)[c(1,nrow(missings))],
                   " (",round(100*apply(missings,1,mean)[c(1,nrow(missings))],0),"%)"),
     cex=1.3)
points(cbind(c(1,nrow(missings)),par()$usr[4]),pch="|")
text(-42,700,labels="(B)",font=2,cex=1.6)

dev.off()
}

####### Data reduction
# Step 1

# Load driver gene list
load("drivers.RData")
length(drivers) # 568 driver genes

# Function to assess whether each alteration involves a driver gene
is.driver<-function(x){sum(startsWith(x,drivers))>0}
is.driver.gene<-sapply(row.names(alteration),is.driver)

# First ten evaluations
is.driver.gene[1:10]
sum(is.driver.gene) # 2567 alterations involving driver genes

# Restrict to driver genes
alteration<-alteration[is.driver.gene,]
freq.alterations<-freq.alterations[is.driver.gene]
prop.alterations<-prop.alterations[is.driver.gene]

# Recap
dim(alteration) # 2567 alterations, 648 cell lines
dim(response) # 265 compounds, 648 cell lines
# Verify vertical matching
all(colnames(alteration)==colnames(response))

####### Data reduction
# Step 2

# Alteration variances
variances<-prop.alterations*(1-prop.alterations)

# Reorder alterations by variance
alteration<-alteration[order(variances),]
freq.alterations<-freq.alterations[order(variances)]
prop.alterations<-prop.alterations[order(variances)]
variances<-sort(variances)

# Set a small proportion
small.p<-0.05
# Compute the corresponding variance
var.min<-small.p*(1-small.p)
var.min

# Restrict above the threshold
alteration<-alteration[variances>var.min,]
prop.alterations<-prop.alterations[variances>var.min]
freq.alterations<-freq.alterations[variances>var.min]
variances<-variances[variances>var.min]

# Proportion of alterations present in each cell line (recalculate)
prop.lines<-apply(alteration,2,mean)
# Reorder cell lines by number of present alterations
alteration<-alteration[,order(prop.lines)]
response<-response[,order(prop.lines)]
prop.lines<-sort(prop.lines)

# Recap
dim(alteration) # 577 alterations, 648 cell lines
dim(response) # 265 compounds, 648 cell lines
# Verify vertical matching
all(colnames(alteration)==colnames(response))

####### Data reduction
# Step 3

# Matrix of squared correlations among alterations
correlations2<-cor(t(alteration))^2
dim(correlations2)
# Distance matrix
distances<-as.dist(1-correlations2)

# Clustering
clustering.alteration<-hclust(d=distances,method="complete")

# Set a high correlation
high.correlation<-0.95
small.distance<-1-high.correlation^2
alteration.clusters<-cutree(clustering.alteration,h=small.distance)

# Reorder alterations by cluster id
alteration<-alteration[order(alteration.clusters),]
prop.alterations<-prop.alterations[order(alteration.clusters)]
freq.alterations<-freq.alterations[order(alteration.clusters)]
variances<-variances[order(alteration.clusters)]
correlations2<-correlations2[order(alteration.clusters),order(alteration.clusters)]
alteration.clusters<-sort(alteration.clusters)
# Number of clusters
length(table(alteration.clusters))
# Frequency distribution of cluster dimensions
table(table(alteration.clusters))

# Store dimension of cluster to which each alteration belongs 
cluster.dim<-rep(table(alteration.clusters),table(alteration.clusters))
names(cluster.dim)<-names(alteration.clusters)
head(cluster.dim)

# See squared correlations within the largest cluster
cluster.id.example<-alteration.clusters[which.max(cluster.dim)]
cluster.id.example # it is cluster 33
alteration.clusters[alteration.clusters==cluster.id.example]
correlations2[alteration.clusters==cluster.id.example,
       alteration.clusters==cluster.id.example]
# See distances within the largest cluster
1-correlations2[alteration.clusters==alteration.clusters[which.max(cluster.dim)],
                   alteration.clusters==alteration.clusters[which.max(cluster.dim)]]

# Keep only the first alteration in each cluster
to.find<-1:max(alteration.clusters)
positions<-match(to.find,alteration.clusters)
alteration<-alteration[positions,]
prop.alterations<-prop.alterations[positions]
freq.alterations<-freq.alterations[positions]
variances<-variances[positions]
correlations2<-correlations2[positions,positions]
# Only keep the vector of correlations between all alteration pairs
correlations2<-correlations2[lower.tri(correlations2,diag=FALSE)]

# Proportion of alterations present in each cell line (recalculate)
prop.lines<-apply(alteration,2,mean)
# Reorder cell lines by number of present alterations
alteration<-alteration[,order(prop.lines)]
response<-response[,order(prop.lines)]
prop.lines<-sort(prop.lines)

# Final recap
dim(alteration) # 501 alterations, 648 cell lines
# Row attributes
length(prop.alterations)
length(freq.alterations)
length(variances)
# Column attributes
length(prop.lines)

dim(response) # 265 compounds, 648 cell lines
# Row attributes
length(sample.sizes)
# Column attributes
length(prop.lines)

# Verify vertical matching
all(colnames(alteration)==colnames(response))

#######################################################
# Data analysis
#######################################################

# Tissues of origin
starts<-stri_locate_first_regex(colnames(alteration),"_")[,1]+1
ends<-nchar(colnames(alteration))
tissues<-substr(colnames(alteration),starts,ends)

tissues.table<-data.frame(Tissue=names(table(tissues)),
                          Freq=as.numeric(table(tissues)),
                          Perc=as.numeric(round(100*prop.table(table(tissues)),1))                          )
tissues.table<-tissues.table[order(tissues.table$Freq,decreasing = TRUE),]
row.names(tissues.table)<-NULL
View(tissues.table)

# Excel export
write.table(tissues.table,"clipboard",quote=FALSE,sep="\t",row.names = FALSE)
# Simply paste (ctrl+v) in an Excel sheet.

####################################################################
# Random Forests

# Skip the execution to save time, and directly load the output
# To re-execute, please select from "Start" to "End" 
# and then press "ctrl+shift+c" to uncomment

# # Start
# {
# # Initialize the outputs
# 
# # CCCs of predicted vs observed responses
# CCC.predictions<-rep(NA,nrow(response))
# # lower limit of the 95% CI
# lower.CCC.predictions<-rep(NA,nrow(response))
# # upper limit of the 95% CI
# upper.CCC.predictions<-rep(NA,nrow(response))
# # stability indicator
# stability<-rep(NA,nrow(response))
# # sample size
# n.training<-rep(NA,nrow(response))
# # CCCs of variances before/after removing missings
# CCC.variances<-rep(NA,nrow(response))
# # CCCs of (squared) correlations before/after removing missings
# CCC.correlations2<-rep(NA,nrow(response))
# # computational time
# times<-rep(NA,nrow(response))
# # variances falling below the threshold after removing missings (per cent)
# low.variances<-rep(NA,nrow(response))
# # (squared) correlations jumping above the threshold after removing missings (per cent mille)
# high.correlations<-rep(NA,nrow(response))
# # alteration permutation importance
# importances<-matrix(NA,nrow(alteration),nrow(response))
# # p-values of alteration permutation importance
# p.importances<-matrix(NA,nrow(alteration),nrow(response))
# 
# ################################################################
# # Set a starting random seed
# set.seed(10)
# 
# # Initialize a progress bar
# pbar <- create_progress_bar('text')
# pbar$init(nrow(response))
# 
# for(id in 1:nrow(response)){
# 
# time.temp<-system.time({
# 
# # Remove missing responses and corresponding predictors
# X<-t(alteration)
# Y<-as.matrix(response[id,])
# X<-X[-which(is.na(Y)),]
# Y<-Y[-which(is.na(Y))]
# 
# # Recalculate variances and squared correlations
# prop.alterations.temp<-apply(X,2,function(x){mean(x=="1")})
# variances.temp<-prop.alterations.temp*(1-prop.alterations.temp)
# correlations2.temp<-cor(X)^2
# correlations2.temp<-correlations2.temp[
#   lower.tri(correlations2.temp,diag=FALSE)]
# 
# # Random forest model
# model<-randomForest(x=X,y=Y,importance=TRUE)
# # Stability
# stability.temp<-mean(tail(diff(model$mse)^2,10))
# # Importances
# importances.temp<-importance(model,type=1,scale=TRUE)
# # P-values
# p.importances.temp<-as.numeric(1-pnorm(importances.temp))
# 
# # Store the model outputs
# importances[,id]<-importances.temp
# p.importances[,id]<-p.importances.temp
# CCC.predictions[id]<-as.numeric(100*CCC(Y,model$predicted)$rho[1])
# lower.CCC.predictions[id]<-as.numeric(100*CCC(Y,model$predicted)$rho[2])
# upper.CCC.predictions[id]<-as.numeric(100*CCC(Y,model$predicted)$rho[3])
# stability[id]<-stability.temp
# n.training[id]<-length(Y)
# CCC.variances[id]<-as.numeric(100*CCC(variances,variances.temp)$rho[1])
# CCC.correlations2[id]<-as.numeric(100*CCC(correlations2,correlations2.temp,na.rm=TRUE)$rho[1])
# low.variances[id]<-round(100*mean(variances.temp<var.min),1)
# high.correlations[id]<-round(100000*mean((1-correlations2.temp)<small.distance,na.rm=TRUE),1)
# })["elapsed"]
# times[id]<-time.temp
# pbar$step()
# }
# 
# # Create an output list to export
# output<-list(
#   importances=importances,
#   p.importances=p.importances,
#   CCC.predictions=CCC.predictions,
#   lower.CCC.predictions=lower.CCC.predictions,
#   upper.CCC.predictions=upper.CCC.predictions,
#   stability=stability,
#   n.training=n.training,
#   CCC.variances=CCC.variances,
#   CCC.correlations2=CCC.correlations2,
#   low.variances=low.variances,
#   high.correlations=high.correlations,
#   times=times)
# 
# save(output,file="output.RData")
# }
# # End

load("output.RData")

# Output names
for(i in 1:2){
  row.names(output[[i]])<-row.names(alteration)
  colnames(output[[i]])<-row.names(response)
  }

for(i in 3:12){
  names(output[[i]])<-row.names(response)
}

# Check missing effects (Figure 2)
{
tiff(width=1.8*4,height=3,"Figure 2.tiff", units = "in", res = 800, compression ="lzw")
par(family="Palatino Linotype")
par(mfrow=c(1,2))
par(mar=c(5,4,2,1))
matplot(output$n.training,
        cbind(output$CCC.variances,output$CCC.correlations2),ylim=c(80,100),
        pch=16,col=c(2,4),xlab="Sample size",ylab="CCC")
legend("bottomright",cex=0.8,legend=c("Variances","Squared correlations"),col=c(2,4),pch=16)
par(xpd=TRUE)
text(213,103,labels="(A)",font=2,cex=1)

issues<-cbind(output$low.variances,output$high.correlations)
matplot(output$n.training,issues,pch=16,col=c(2,4),ylim=c(0,50),
        xlab="Sample size",ylab="Frequency",cex.axis=0.9)
legend("topright",cex=0.8,legend=c("Low variances (%)",
                           paste0("High correlations (pcm)")),
       col=c(2,4),pch=16)
text(213,57,labels="(B)",font=2,cex=1)

dev.off()
}

# P-values in Figure 3
p.value<-round(summary(lm(output$times~output$n.training))[[4]][2,4],3)
p.value

# Check computational times and model stability (Figure 3)
{
  tiff(width=3.8*2,height=3.2,"Figure 3.tiff", units = "in", res = 800, compression ="lzw")
  par(family="Palatino Linotype")
  par(mfrow=c(1,2))
  par(mar=c(5,5,1.5,1.5))
  plot(output$n.training,output$times,
       ylab="Elapsed time (seconds)",
       xlab="Sample size",
       pch=16,cex=0.8,col=4,ylim=c(0,20))
  total.time<-round(sum(output$times)/60,0)
  text(210,17,labels=bquote(paste("Total elapsed time",phantom()%~~%phantom(),
                                  "",.(total.time)," minutes")),pos=4,cex=0.80)

  text(400,5,labels=expression(paste(italic(p),"-value < 0.001")),cex=0.80,pos=4)
  abline(lm(output$times~output$n.training),col=2,lwd=2)
  par(xpd=TRUE)
  text(220,22.5,labels="(A)",font=2,cex=1)
  
  par(mar=c(1.5,5,1.5,1.5))
  boxplot(output$stability,
          ylab="Mean of last 10 squared error changes",
          col=4,cex.lab=0.9,axes=FALSE)
  box()
  axis(side=2,cex.axis=0.9,at=seq(0,1.2*10^-9,l=7),
       labels=c(0,
                "",
                quote(paste("4 \u00D7 10"^"\u221210")),
                "",
                expression(paste("8 \u00D7 10"^"\u221210")),
                "",
                expression(paste("1.2 \u00D7 10"^"\u22129")))
       )
  text(0.51,10^-8.87,labels="(B)",font=2,cex=1)
  dev.off()
}

################ Predictive power
# CCC benchmarks: 
# <=0 "no concordance"
# 0 to 0.20 "poor concordance" 
# 0.21 to 0.40 "fair concordance" 
# 0.41 to 0.60 "moderate concordance" 
# 0.61 to 0.80 "substantial concordance" 
# >0.80 "excellent concordance"
CCC.classes<-cut(output$CCC.predictions,c(-100,0,20,40,60,80,100))
levels(CCC.classes)
levels(CCC.classes)<-c("None\n\u22640","Poor\n(0-20]","Fair\n(20-40]",
                      "Moderate\n(40-60]","Substantial\n(60-80]","Excellent\n(80-100]")
table(CCC.classes)
perc.CCC<-round(100*prop.table(table(CCC.classes)),1)

# P-values in Figure 4
p.value1<-round(summary(lm(output$CCC.predictions~output$n.training))[[4]][2,4],3)
p.value2<-round(summary(lm(output$CCC.predictions~
                             rowMeans(response,na.rm=TRUE)))[[4]][2,4],3)
p.value1
p.value2

# See the CCCs (Figure 4)
{
tiff(width=6,height=6,"Figure 4.tiff", units = "in", res = 800, compression ="lzw")
par(family="Palatino Linotype")
layout(matrix(c(1,1,2,3),2,2,byrow=TRUE),heights =c(1.2,1))

par(mar=c(4,5,1.5,0))
par(xpd=TRUE)
fig<-barplot(perc.CCC,ylim=c(0,80),ylab="Frequency (%)",
             col=4,cex.axis = 1.2,cex.lab=1.2,
             names.arg=c("","","","","",""))
text(fig,rep(-9,6),labels=levels(CCC.classes),cex=1.1)
text(fig,perc.CCC+4,labels=paste0(perc.CCC,"%"),cex=1.1)
text(-1,78,labels="(A)",font=2,cex=1.4)

par(mar=c(5,4.5,1,1.5))
plot(output$n.training,output$CCC.predictions,col=4,pch=16,cex.axis=1.2,
     xlab="Sample size",ylab="CCC",cex.lab=1.2)
text(75,62,labels="(B)",font=2,cex=1.4)
par(xpd=FALSE)
abline(lm(output$CCC.predictions~output$n.training),col=2,lwd=2)
abline(h=c(0,20),lty=2,lwd=2)
text(350,60,labels=expression(paste(italic(p),"-value = 0.020")),cex=1.2)

plot(rowMeans(response,na.rm=TRUE),output$CCC.predictions,pch=16,col=4,cex.axis=1.2,
     xlab="Average compound AUC",ylab="CCC",cex.lab=1.2)
abline(h=c(0,20),lty=2,lwd=2)
abline(lm(output$CCC.predictions~rowMeans(response,na.rm=TRUE)),col=2,lwd=2)

text(0.5,60,labels=expression(paste(italic(p),"-value < 0.001")),cex=1.2)
par(xpd=TRUE)
text(-0.05,62,labels="(C)",font=2,cex=1.4)

dev.off()
}

#### Output report 1
report1<-data.frame(Compound=row.names(response),
                   CCC=round(output$CCC.predictions,1),
                   CCC_lower=round(output$lower.CCC.predictions,1),
                   CCC_upper=round(output$upper.CCC.predictions,1),
                   mean_AUC=round(apply(response,1,mean,na.rm=TRUE),3),
                   sample.size=sample.sizes)

# Adjusted p-values
bonferroni.p.importances<-apply(output$p.importances,2,bonferroni)
# Define statistical significance
significant.p<-0.005
significant.alterations<-bonferroni.p.importances<significant.p

# Which are the significant alterations for each compound?
# (ordered by importance)
influent.alterations<-list(NA)
for(i in 1:nrow(report1))
  influent.alterations[[i]]<-row.names(alteration)[significant.alterations[,i]][
  order(output$importances[significant.alterations[,i],i],decreasing = TRUE)]
# How many significant alterations for each compound?
report1$n.influent<-sapply(influent.alterations,length)

# Store their names
max.columns<-max(report1$n.influent)
strings<-matrix(NA,nrow(report1),max.columns)
for(i in 1:nrow(report1))
  try(strings[i,1:length(influent.alterations[[i]])]<-
        influent.alterations[[i]],silent=TRUE)
colnames(strings)<-paste0("Alt",1:max.columns)
report1<-cbind(report1,strings)

## Reorder by CCC value
new.order<-order(report1$CCC,decreasing = TRUE)
report1<-report1[new.order,]
influent.alterations<-influent.alterations[new.order]
row.names(report1)<-NULL

# Responses with CCC>20
fair<-which(report1$CCC_lower>20)
report1<-report1[fair,]
influent.alterations<-influent.alterations[fair]
exclude<-which(apply(report1,2,function(x){sum(!is.na(x))})==0)
if(length(exclude)>0)
  report1<-report1[,-exclude]

View(report1)

# Excel export
write.table(report1,"clipboard",quote=FALSE,sep="\t",row.names = FALSE)
# Simply paste (ctrl+v) in an Excel sheet.

######################################################################
# How many times are alterations important for these compounds?
# Do they belong to some alteration cluster?

#### Output report 2
influent<-unlist(influent.alterations)
more.influent<-names(sort(table(influent),decreasing=TRUE))
freq.influent<-as.numeric(sort(table(influent),decreasing=TRUE))
report2<-data.frame(Alteration=more.influent,
                    Frequency=freq.influent,
                    Cluster_ID=alteration.clusters[more.influent],
                    Cluster_size=cluster.dim[more.influent]
                    )
row.names(report2)<-NULL

# Which are the compounds with CCC>20 influenced by each alteration?
# (ordered by CCC)
influenced.compounds<-list(NA)
for(i in 1:length(more.influent)){
influenced.compounds.i<-apply(report1,1,function(x){more.influent[i] %in% x})
influenced.compounds[[i]]<-report1$Compound[influenced.compounds.i]
}

# Store their names
max.columns<-max(freq.influent)
strings<-matrix(NA,length(more.influent),max.columns)
for(i in 1:length(more.influent))
  strings[i,1:freq.influent[i]]<-influenced.compounds[[i]]
colnames(strings)<-paste0("Comp",1:max.columns)

report2<-cbind(report2,strings)

View(report2)

# Excel export
write.table(report2,"clipboard",quote=FALSE,sep="\t",row.names = FALSE)
# Simply paste (ctrl+v) in an Excel sheet.

# Clusters
clusters<-which(report2$Cluster_size>1)

alteration.clusters[alteration.clusters %in%
                      report2$Cluster_ID[clusters]]

################################################################
# Discovering interactions example 1
################################################################

# Compounds of interest
c1<-"PLX4720"
c2<-"Nutlin-3a"

# Alterations of interest
a1<-"BRAF.V600E_MUT"
a2<-"TP53_MUT"

################################################
# Extract data of interest
y1<-response[c1,]
y2<-response[c2,]
x1<-alteration[a1,]
x2<-alteration[a2,]

# Create a long dataset
long<-data.frame(y=c(y1,y2))
long$x1<-c(x1,x1)
long$x2<-c(x2,x2)
long$compound<-factor(c(rep(c1,length(y1)),
                        rep(c2,length(y2))),
                      levels=c(c1,c2))
long<-long[-which(is.na(long$y)),]
head(long)

###################################################
# Create x combinations
long$xx<-factor(long$x1):factor(long$x2)

# Check conditional distributions
boxplot(long$y~long$xx:long$compound)
# Normalize
long$y<-my.logit(long$y)
boxplot(long$y~long$xx:long$compound)

# Add overall
levels(long$xx)<-c(levels(long$xx),"Overall")
table(long$xx)
overall<-long
overall$xx<-factor("Overall",levels=levels(long$xx))
table(overall$xx)
long<-rbind(overall,long)
# Relevel
levels(long$xx)
long$xx<-factor(long$xx,levels=levels(long$xx)[c(5,1,3,2,4)])
table(long$xx)

# Sample sizes
group.sizes<-tapply(long$y,list(long$compound,long$xx),length)
group.sizes
# Conditional means
group.means<-tapply(long$y,list(long$compound,long$xx),mean)
group.means

# Group names
group.names<-c(paste0("Overall\n(",group.sizes[1,1]," vs. ",group.sizes[2,1]," cell lines)"),
               paste0(a1,"-\n",a2,"-\n(",group.sizes[1,2]," vs. ",group.sizes[2,2]," cell lines)"),
               paste0(a1,"+\n",a2,"-\n(",group.sizes[1,3]," vs. ",group.sizes[2,3]," cell lines)"),
               paste0(a1,"-\n",a2,"+\n(",group.sizes[1,4]," vs. ",group.sizes[2,4]," cell lines)"),
               paste0(a1,"+\n",a2,"+\n(",group.sizes[1,5]," vs. ",group.sizes[2,5]," cell lines)")
               )

# t-tests (p-values)
pp<-rep(NA,nlevels(long$xx))
for(i in 1:nlevels(long$xx))
  try(pp[i]<-round(t.test(long$y[long$xx==levels(long$xx)[i]]~
                            long$compound[long$xx==levels(long$xx)[i]])$p.value,3),silent=TRUE)
pp
pp<-c(expression(paste(italic(p),"-value = 0.869")),
      expression(paste(italic(p),"-value < 0.001")),
      expression(paste(italic(p),"-value = 0.014")),
      expression(paste(italic(p),"-value < 0.001")),
      expression(paste(italic(p),"-value < 0.001")))
pp

# Group max
group.max<-tapply(long$y,long$xx,max)

# Visualize statistical interactions (Figure 5)
space<-0.15
{
tiff(width=6*1.2,height=2.5*1.2,"Figure 5.tiff", 
     units = "in", res = 800, compression ="lzw")
par(family="Palatino Linotype")
par(mar=c(3,5,2,1))
boxplot(y~xx, data = subset(long,compound=="PLX4720"),axes=FALSE,
        boxwex = 0.25, 
        at = 1:5 - space,
        col = 2,
        xlab = "",
        ylab = "logit(AUC)",
        xlim = c(0.5, 5.5), 
        ylim = range(long$y)+c(-0.3,1),
        yaxs = "i")
boxplot(y~xx, data = subset(long,compound=="Nutlin-3a"),add = TRUE,
        axes=FALSE, 
        boxwex = 0.25,
        at = 1:5 + space,
        col = 4)
axis(side=1,tck=-0.03,at=0:5,cex.axis=0.7,
     labels=rep("",6))
par(xpd=TRUE)
text(1:5,rep((range(long$y)+c(-0.3,1))[1]-1.5,5),
     labels=group.names,cex=0.6)
axis(side=2,tck=-0.03,at=-1:7,labels=-1:7)
text(1:5,group.max+0.8,labels=pp,cex=0.8)
legend(3,(range(long$y)+c(-0.3,1))[2]+1.5,xjust=0.5,
       c("PLX4720", "Nutlin-3a"),horiz = TRUE,bty="n",
       fill = c(2, 4),cex=0.8)
points(1:5-space,group.means[1,],pch=15)
points(1:5+space,group.means[2,],pch=15)
dev.off()
}

# Formal test for interactions
additive.model<-lm(y~xx+compound,data=subset(long,xx!="Overall"))
multiplicative.model<-lm(y~xx*compound,data=subset(long,xx!="Overall"))

summary(multiplicative.model)
anova(additive.model,multiplicative.model)

################################################################
# Discovering interactions example 2
################################################################

# Compounds of interest
c1<-"Dabrafenib"
c2<-"Afatinib (rescreen)"

# Alterations of interest
a1<-"BRAF.V600E_MUT"
a2<-"IKZF3_AMP"

################################################
# Extract data of interest
y1<-response[c1,]
y2<-response[c2,]
x1<-alteration[a1,]
x2<-alteration[a2,]

# Create a long dataset
long<-data.frame(y=c(y1,y2))
long$x1<-c(x1,x1)
long$x2<-c(x2,x2)
long$compound<-factor(c(rep(c1,length(y1)),
                        rep(c2,length(y2))),
                      levels=c(c1,c2))
long<-long[-which(is.na(long$y)),]

###################################################
# Create x combinations
long$xx<-factor(long$x1):factor(long$x2)

# Check conditional distributions
boxplot(long$y~long$xx:long$compound)
# Normalize
long$y<-my.logit(long$y)
boxplot(long$y~long$xx:long$compound)

# Add overall
levels(long$xx)<-c(levels(long$xx),"Overall")
table(long$xx)
overall<-long
overall$xx<-factor("Overall",levels=levels(long$xx))
table(overall$xx)
long<-rbind(overall,long)
# Relevel
levels(long$xx)
long$xx<-factor(long$xx,levels=levels(long$xx)[c(5,1,3,2,4)])
table(long$xx)

# Sample sizes
group.sizes<-tapply(long$y,list(long$compound,long$xx),length)
group.sizes[which(is.na(group.sizes))]<-0
group.sizes
# Conditional means
group.means<-tapply(long$y,list(long$compound,long$xx),mean)
group.means

# Group names
group.names<-c(paste0("Overall\n(",group.sizes[1,1]," vs. ",group.sizes[2,1]," cell lines)"),
               paste0(a1,"-\n",a2,"-\n(",group.sizes[1,2]," vs. ",group.sizes[2,2]," cell lines)"),
               paste0(a1,"+\n",a2,"-\n(",group.sizes[1,3]," vs. ",group.sizes[2,3]," cell lines)"),
               paste0(a1,"-\n",a2,"+\n(",group.sizes[1,4]," vs. ",group.sizes[2,4]," cell lines)"),
               paste0(a1,"+\n",a2,"+\n(",group.sizes[1,5]," vs. ",group.sizes[2,5]," cell lines)")
)

# t-tests (p-values)
pp<-rep(NA,nlevels(long$xx))
for(i in 1:nlevels(long$xx))
  try(pp[i]<-round(t.test(long$y[long$xx==levels(long$xx)[i]]~
                            long$compound[long$xx==levels(long$xx)[i]])$p.value,3),silent=TRUE)
pp
pp<-c(expression(paste(italic(p),"-value = 0.062")),
      expression(paste(italic(p),"-value = 0.962")),
      expression(paste(italic(p),"-value < 0.001")),
      expression(paste(italic(p),"-value < 0.001")),
      NA)
pp

# Group max
group.max<-tapply(long$y,long$xx,max)


# Visualize statistical interactions (Figure 6)
space<-0.15
{
tiff(width=6*1.2,height=2.5*1.2,"Figure 6.tiff", 
     units = "in", res = 800, compression ="lzw")
par(family="Palatino Linotype")
par(mar=c(3,5,2,1))
boxplot(y~xx, data = subset(long,compound=="Dabrafenib"),axes=FALSE,
        boxwex = 0.25, 
        at = 1:5 - space,
        col = 2,
        xlab = "",
        ylab = "logit(AUC)",
        xlim = c(0.5, 5.5), 
        ylim = range(long$y)+c(-0.4,1.7),
        yaxs = "i")
boxplot(y~xx, data = subset(long,compound=="Afatinib (rescreen)"),add = TRUE,
        axes=FALSE, 
        boxwex = 0.25,
        at = 1:5 + space,
        col = 4)
axis(side=1,tck=-0.03,at=0:5,cex.axis=0.7,
     labels=rep("",6))
par(xpd=TRUE)
text(1:5,rep((range(long$y)+c(-0.4,1.7))[1]-2,5),
     labels=group.names,cex=0.6)
axis(side=2,tck=-0.03)
text(1:5,group.max+1,labels=pp,cex=0.8)
legend(3,(range(long$y)+c(-0.4,1.7))[2]+2,xjust=0.5,text.width=c(0.6,0),
       c("Dabrafenib", "Afatinib (rescreen)"),horiz = TRUE,bty="n",
       fill = c(2, 4),cex=0.8)
points(1:5-space,group.means[1,],pch=15)
points(1:5+space,group.means[2,],pch=15)
dev.off()
}

# Formal test for interactions
additive.model<-lm(y~xx+compound,data=subset(long,xx!="Overall"))
multiplicative.model<-lm(y~xx*compound,data=subset(long,xx!="Overall"))

summary(multiplicative.model)
anova(additive.model,multiplicative.model)

#######################################################
######### Data not shown in the paper

# Verify single sensitivity / resistance
boxplot(response["Afatinib (rescreen)",]~alteration["ERBB2_AMP",]) # sensitivity
boxplot(response["Afatinib (rescreen)",]~alteration["EGFR_AMP",]) # sensitivity
boxplot(response["Nutlin-3a",]~alteration["MAP2K4_DEL",]) # resistance

# Power calculations
total.n<-trunc(mean(sample.sizes))
total.n # mean sample size
effect<-cohen.ES(test="t",size="medium")$effect

small.prop<-0.05 # our choice (0.70 power)
# small.prop<-0.065 # 0.80 power

power_t_test(n=small.prop*total.n,
             delta=effect, 
             sig.level=0.05,
             power=NULL, 
             ratio=(1-small.prop)/small.prop,
             type="two.sample", 
             alternative="two.sided", 
             df.method="classical")
